// SPDX-License-Identifier: GPL-2.0
// Copyright (C) 2021 Jakub Horak
#include <util/twi.h>

#define F_SCL 888000

static void
i2c_init(void)
{
	/* SCL_freq = CPU_freq / (16 + 2*TWBR*Prescaler) */
	/* TWBR = (CPU_freq / SCL_Freq - 16) / (2 * Prescaler) */
	 
	/* set prescaler = 1 */
	TWSR = 0;
	/* set divider to F_SCL */
	TWBR = F_CPU / F_SCL / 2 - 8;
	/* enable */
	TWCR = _BV(TWEN);
}

static inline byte
i2c_wait(void)
{
	while (!(TWCR & _BV(TWINT)))
		;
	return TWSR & 0xf8;
}

static inline void
i2c_continue(void)
{
	TWCR = _BV(TWINT) | _BV(TWEN) | _BV(TWEA);
}

static byte
i2c_start_transmit(byte address)
{
	byte stat;

	TWCR = _BV(TWEN);

	/* step: start condition */
	TWCR |= _BV(TWSTA) | _BV(TWINT);
	stat = i2c_wait();
	if (stat != TW_START)
		return 1;

	/* step: write address */
	TWDR = (address << 1) | TW_WRITE;
	i2c_continue();
	stat = i2c_wait();
	if (stat != TW_MT_SLA_ACK)
		return 2;

	return 0;
}

static inline byte
i2c_send_byte(byte x)
{
	byte stat;

	/* step: xmit data */
	TWDR = x;
	i2c_continue();
	stat = i2c_wait();
	if (stat != TW_MT_DATA_ACK)
		return 3;

	return 0;
}

static void
i2c_stop(void)
{
	/* step: stop condition */
	TWCR |= _BV(TWINT) | _BV(TWSTO);
	/* wait for it to clear */
	while (TWCR & _BV(TWSTO))
		;
}
