// SPDX-License-Identifier: GPL-2.0
// Copyright (C) 2021 Jakub Horak
extern const byte ssd1306_init_seq[];
extern const byte ssd1306_page_start[];

enum {
	ScreenWidth = 128,
	ScreenHeight = 64,
};

enum {
	SSD1306_I2CAddr = 0x3c,
	SSD1306_ModeCmd = 0x00,
	SSD1306_ModeData = 0x40,

};

static byte 
ssd1306_send_command(const byte *ptr)
{
	byte ret;

	ret = i2c_start_transmit(SSD1306_I2CAddr);
	if (ret)
		return ret;
	ret = i2c_send_byte(SSD1306_ModeCmd);
	if (ret)
		return ret;

	for (byte i=0;; i++) {
		byte cmd = pgm_read_byte_near(&ptr[i]);
		if (cmd == 0xff)
			break;
		ret = i2c_send_byte(cmd);
		if (ret)
			return ret;
	}
	i2c_stop();
	return 0;
}

static byte
ssd1306_send_screen(const byte *screen)
{
	byte ret;

	ret = ssd1306_send_command(ssd1306_page_start);
	if (ret)
		return ret;
	ret = i2c_start_transmit(SSD1306_I2CAddr);
	if (ret)
		return ret;
	ret = i2c_send_byte(SSD1306_ModeData);
	if (ret)
		return ret;

	for (byte page=0; page<ScreenHeight/8; page++) {
		for (byte col=0; col<ScreenWidth; col++) {
			byte c = screen[ScreenWidth*page + col];

			ret = i2c_send_byte(c);
			if (ret)
				return ret;
		}
	}
	i2c_stop();
	return 0;
}

static byte
ssd1306_init(void)
{
	return ssd1306_send_command(ssd1306_init_seq);
}
